from abc import ABCMeta
from typing import Dict

from .Command import Command
from ..state import World, GameState


class Logic:

    def __init__(self, state: GameState):
        self.__commands: Dict[int, Command] = {}
        self.__state = state

    @property
    def state(self) -> GameState:
        return self.__state

    @property
    def world(self) -> World:
        return self.__state.world

    def addCommand(self, command: Command):
        self.__commands[command.priority()] = command

    def executeCommands(self):
        commands = self.__commands.copy()
        self.__commands.clear()
        priorities = sorted(commands.keys())
        for priority in priorities:
            command = commands[priority]
            if not command.check(self):
                continue
            command.execute(self)

    def getSetLayerCellCommand(self, layer: str) -> ABCMeta:

        from core.logic.command.layer.SetGroundCellCommand import SetGroundCellCommand
        from core.logic.command.layer.SetImpassableCellCommand import SetImpassableCellCommand
        from core.logic.command.layer.SetObjectsCellCommand import SetObjectsCellCommand
        from core.logic.command.layer.SetUnitsCellCommand import SetUnitsCellCommand
        setLayerValueCommand = {
            "ground": SetGroundCellCommand,
            "impassable": SetImpassableCellCommand,
            "objects": SetObjectsCellCommand,
            "units": SetUnitsCellCommand,
        }
        return setLayerValueCommand[layer]

