from __future__ import annotations

from typing import Tuple, Optional, List

from core.constants import CellValue
from core.logic import DistanceMap
from core.logic.Command import Command, WORLD_PRIORITY, WORLD_MAX_WIDTH
from core.logic.Logic import Logic


class MoveUnitCommand(Command):

    def __init__(self, fromCell: Tuple[int, int], toCell: Tuple[int, int]):
        self.__fromCell = fromCell
        self.__toCell = toCell
        self.__path: List[Tuple[int, int]] = []

    def priority(self) -> int:
        return WORLD_PRIORITY + self.__fromCell[0] + self.__fromCell[1] * WORLD_MAX_WIDTH

    def check(self, logic: Logic) -> bool:
        if self.__fromCell == self.__toCell:
            return False

        world = logic.world
        if not world.contains(self.__fromCell):
            return False
        if not world.contains(self.__toCell):
            return False

        unit = world.units.getUnit(self.__fromCell)
        if unit is None:
            return False
        #if unit.playerId != logic.state.currentPlayerId:
        #    return False

        # Compute best direction (if any)
        fromX, fromY = self.__fromCell
        toX, toY = self.__toCell
        radius = 10
        area = (
            min(fromX, toX) - radius, min(fromY, toY) - radius,
            max(fromX, toX) + radius + 1, max(fromY, toY) + radius + 1
        )
        distanceMap = DistanceMap(world, area)
        distanceMap.compute(self.__fromCell)
        self.__path, costs = distanceMap.getPath(self.__toCell)
        if len(self.__path) == 0:
            return False
        return True

    def execute(self, logic: Logic):
        unitsLayer = logic.world.units
        unit = unitsLayer.getUnit(self.__fromCell)
        unitsLayer.setUnit(self.__fromCell, CellValue.NONE)
        targetCell = self.__path[-1]
        unitsLayer.setUnit(targetCell, CellValue.UNITS_UNIT, unit)
        unitsLayer.notifyCellChanged(self.__fromCell)
        unitsLayer.notifyCellChanged(targetCell)

        if len(self.__path) > 1:
            command = MoveUnitCommand(targetCell, self.__toCell)
            logic.addCommand(command)

