import logging
import os.path
from typing import Dict, Any, cast, Optional

from .World import World
from ..IDataTransfer import IDataTransfer


class GameState(IDataTransfer):
    def __init__(self, world: Optional[World] = None):
        self.__currentPlayerId = 1
        if world is None:
            world = World((10, 10))
        self.__world = world

    def __eq__(self, other) -> bool:
        if not isinstance(other, GameState):
            return False
        state = cast(GameState, other)
        if self.__currentPlayerId != state.__currentPlayerId:
            return False
        if self.__world != state.__world:
            return False
        return True

    @property
    def world(self) -> World:
        return self.__world

    @property
    def currentPlayerId(self) -> int:
        return self.__currentPlayerId

    # Data transfer

    def load(self, fileName: str):
        logging.info(f"Load state from {fileName}...")
        ext = os.path.splitext(fileName)[1]
        if ext == ".json":
            import json
            with open(fileName, encoding='utf-8') as file:
                data = json.load(file)
        elif ext == ".pickle":
            import pickle
            with open(fileName, 'rb') as bfile:
                data = pickle.load(bfile)
        else:
            raise ValueError(f"Invalid file extension {ext}")

        # Check
        try:
            state = GameState()
            state.takeData(data)
        except Exception as ex:
            raise ValueError(f"Error in file {fileName}")

        # Take data
        self.takeData(data)

    def save(self, fileName: str):
        # Build data
        data = self.gatherData()

        # Save
        logging.info(f"Save state to {fileName}...")
        ext = os.path.splitext(fileName)[1]
        if ext == ".json":
            import json
            with open(fileName, 'w', encoding='utf-8') as file:
                json.dump(data, file, ensure_ascii=False, indent=4)
        elif ext == ".pickle":
            import pickle
            with open(fileName, 'wb') as bfile:
                pickle.dump(data, bfile, protocol=4)
        else:
            raise ValueError(f"Invalid file extension {ext}")

    def gatherData(self) -> Dict[str, Any]:
        return {
            "currentPlayerId": self.__currentPlayerId,
            "world": self.__world.gatherData()
        }

    def takeData(self, data: Dict[str, Any]):
        self.__currentPlayerId = int(data["currentPlayerId"])
        self.__world = World((1, 1))
        self.__world.takeData(data["world"])

