from typing import Callable, NoReturn

from pygame.surface import Surface

from ..Component import Component
from ...IUIEventHandler import IUIEventHandler
from ...Mouse import Mouse
from ...MouseWheel import MouseWheel
from ...theme.Theme import Theme


class Button(Component, IUIEventHandler):

    def __init__(self, theme: Theme,
                 surface: Surface,
                 action: Callable[[Mouse], NoReturn]):
        self.__tile = surface
        self.__action = action
        super().__init__(theme, surface.get_size())

    def render(self, surface: Surface):
        surface.blit(self.__tile, self.topLeft)

    # UI Event Handler

    def mouseEnter(self, mouse: Mouse) -> bool:
        return True

    def mouseLeave(self) -> bool:
        return True

    def mouseButtonDown(self, mouse: Mouse) -> bool:
        self.__action(mouse)
        return True

    def mouseButtonUp(self, mouse: Mouse) -> bool:
        return True

    def mouseWheel(self, mouse: Mouse, wheel: MouseWheel) -> bool:
        return True

    def mouseMove(self, mouse: Mouse) -> bool:
        return True
