from typing import Any, Tuple, Union, Dict

from tools.vector import vectorSubDiv2I
from ..Tileset import Tileset
from ui.layer.SelectionValue import SelectionValue

selectionTiledef: Dict[str, Any] = {
    "imageFile": "toen/selection.png",
    "tileSize": (16, 16),
    "tiles": {
        SelectionValue.NONE: [(0, 0)],
        SelectionValue.SELECT: [(0, 1, 3, 3), (3, 1, 3, 3), (6, 1, 3, 3), (9, 1, 3, 3), (12, 1, 3, 3)],
        SelectionValue.NUMBER_FIRST: (0, 4)
    },
}


def initSelectionTiledef(tileset: Tileset):
    from ...component.text.token.TextStyle import TextStyle

    textStyle = TextStyle("small", "white")
    tileSize = selectionTiledef["tileSize"]
    tiles = selectionTiledef["tiles"]
    x, y = tiles[SelectionValue.NUMBER_FIRST]
    theme = tileset.theme
    surface = tileset.surface
    for value in range(SelectionValue.NUMBER_FIRST, SelectionValue.NUMBER_LAST):
        # Register tile
        tileset.addTile(value, (x, y))

        # Render
        number = value - SelectionValue.NUMBER_FIRST
        textSurface = textStyle.render(theme, f"{number}")
        rect = tileset.getTileRect(value)
        shift = vectorSubDiv2I(tileSize, textSurface.get_size())
        surface.blit(textSurface, rect.move(shift[0], shift[1]))

        # Update coords
        x += 1
        if (x * tileSize[0]) >= surface.get_width():
            x = 0
            y += 1
